/*
	- Copy this script and the sub folder 'Includes' in a separate directory
	- Start the script via the MemBrain menu <Scripting><Execuet Script...>
	- Select csv file you want to create a net for
	- Specifiy the number of output columns contained in the csv via script driven dialog - the rest will be assumed to be inputs
	
	--> A feed forwar net will be create for you in alignment with the data in the csv file.
	--> Ensure that the csv file contains the names of the input and output columns in its first row
	--> Ensure that in the csv file first all input columns are specified followed by all output columns
*/

// Default number of outputs assumed when asking the user for this information
const uint DEFAULT_OUT_COUNT = 1;

// Default number of hiodden layers for user dialog
const uint DEFAULT_HID_LAYER_COUNT = 1;
        

#include "Includes\\NetEditor.as"

// static objects created from script classes
NetEditor gNetEditor;               // The net editor object


/*
    Main program (Script entry point)
*/
void main()
{
    // Do some initialization
    Init();
}


/*
    Initialization after the script has been started.
*/
void Init()
{
    // Clear any data currently available in the Lesson Editor
    SetLessonCount(1);
    ClearLesson();
    
    // Hide all currently not required windows
    ShowLessonEditor(false);
    ShowErrorViewer(false);
    ShowPatternErrorViewer(false);
    
    // Load the csv to create the net for into lesson #1
    LoadCsv();
    // Now create the net
    CreateNet();
    AssertTrue(gNetEditor.NormalizeNetWithActiveLesson(), "Unable to normalize net inputs and/or outputs");
}

/*
    Load an existing csv file to generate neural net for.
*/
void LoadCsv()
{
    string path;
    uint inCount = 0;
    uint outCount = 0;
    
    int temp;
    
    AssertTrue(FileOpenDlg("Select csv file to create net for", "csv", "", path) == IDOK, "No csv file selected");
    
    uint csvColumns = GetNumberCsvColumns(path);
    
    AssertTrue(csvColumns > 1, "No or too few data columns found in csv file.\r\nAt least one input and one output column required.");
    int defaultOutputCount = DEFAULT_OUT_COUNT;
    if (defaultOutputCount >= int(csvColumns))
    {
    	defaultOutputCount = csvColumns - 1;
    }
    AssertTrue(UserInput("Specify number of output columns in the csv file", defaultOutputCount, temp) == IDOK, "Number of outputs must be specified");
    AssertTrue((temp >= 1) && (temp < int(csvColumns)), "Invalid number of outputs specified");
    outCount = uint(temp);
    
    inCount = csvColumns - outCount;
    
    SetLessonInputCount(inCount);
    SetLessonOutputCount(outCount);
    ImportLessonRaw(path);
}


/*
    Create a new net that fits to the experiment's data layout.
*/
void CreateNet()
{
    uint inCount = GetLessonInputCount();
    uint outCount = GetLessonOutputCount();
    uint hidLayerCount = 0;
    int temp;
    
    AssertTrue(UserInput("Specify number of hidden layers for the net", DEFAULT_HID_LAYER_COUNT, temp) == IDOK, "Number of hidden layers must be spoecified");
    AssertTrue(temp >= 0, "Number of hidden layers must be >= 0");
    hidLayerCount = uint(temp);
    
    uint hidNeuronCount = inCount / 3;
    
    if (hidNeuronCount < 1)
    {
    	hidNeuronCount = 1;
    }
    
    if (hidLayerCount > 0)
    {
    	AssertTrue(UserInput("Specify number of neurons per hidden layer for the net", int(hidNeuronCount), temp) == IDOK, "Number of neurons per hidden layer must be spoecified");
    	AssertTrue(temp > 0, "Number of neurons per hidden layer must be > 0");
    	hidNeuronCount = uint(temp);
    }
    
    // Create the net's input and output neurons and name the output neurons (with the participants' names)
    gNetEditor.CreateNet(inCount, outCount, hidLayerCount);
    NamesToNet();
    
    array<uint16> hiddenCounts;
    
    hiddenCounts.resize(hidLayerCount);
    
    for (uint i = 0; i < hidLayerCount; i++)
    {
    	hiddenCounts[i] = hidNeuronCount;
    }
    
   
    // Create the desired hidden layers, randomize and save the net.
    gNetEditor.CreateHiddenLayers(hiddenCounts);
    RandomizeNet();
}



/*
    This function finds the index in the given array that represents the maximum value 
    in the array.
    If the array contains all equal values then 0 is returned as maximum index.
*/
uint FindMaxIdx(uint[] &in arr)
{
    uint maxIdx = 0;
    uint max = 0;
    uint len = arr.length();
    
    for (uint i = 0; i < len; i++)
    {
        if (arr[i] > max)
        {
            max = arr[i];
            maxIdx = i;
        }
    }
    
    return maxIdx;
}

/*
    Check how many csv columns there are in the given csv file
*/
uint GetNumberCsvColumns(const string &in fileName)
{
    file csvFile;
    uint columns = 0;
    string listSep;
    string decSep;
    
    // Get MemBrain's csv file separators
    GetCsvFileSeparators(listSep, decSep);
    
    // Open the file
    bool ok = csvFile.Open(fileName, FILE_MODE_READ | FILE_SHARE_DENY_WRITE);
    
    if (ok)
    {
        string s;
        int idx = 0;
        
        // Read first line
        ok = csvFile.Read(s);
        int len = s.length();
        
        // Now count number of list separators in the line
        if (len > 0)
        {
        	columns = 1;
        	while (idx >= 0)
        	{
        		idx = s.findFirst(listSep, idx);
        		// Found one and there is data following
        		if ((idx >= 0) && (idx + 1 < len))
        		{
        			columns++;
        			idx++;
        		} 
        	}
        }
        
        csvFile.Close();
    }
    
    return columns;
}


// Abort the script with an error message
void AbortWithMessage(string message)
{
	MessageBox("Script aborts due to error!\r\n\r\n" + message);
	AbortScript();
}

// Check a boolean condition. Abort the script with the given error message in case the condition is false.
void AssertTrue(bool condition, string errorMessage)
{
	if (!condition)
	{
		AbortWithMessage(errorMessage);
	}
}






